import { Button, Input, Space } from 'antd';
import { SearchOutlined, PlusOutlined } from '@ant-design/icons';
import React from 'react';
import { ImportExportButtons } from './ImportExportButtons';

// Define or import the Staff type
interface Staff {
    id: number;
    name: string;
    username: string;
}

interface SearchBarProps {
    searchText: string;
    onSearchTextChange: (text: string) => void;
    onSearch: () => void;
    onAdd: () => void;
    onImportSuccess: () => void;
    data: Staff[];
}

export const SearchBar: React.FC<SearchBarProps> = ({
    searchText,
    onSearchTextChange,
    onSearch,
    onAdd,
    onImportSuccess,
    data,
}) => {
    return (
        <div className="mb-4 flex justify-between">
      <Space>
        <Input
          placeholder="搜索员工姓名或用户名"
          value={searchText}
          onChange={(e) => onSearchTextChange(e.target.value)}
          onPressEnter={onSearch}
          prefix={<SearchOutlined />}
        />
        <Button type="primary" onClick={onSearch}>搜索</Button>
      </Space>
      <Space>
        <ImportExportButtons 
          onImportSuccess={onImportSuccess}
          data={data}
        />
        <Button type="primary" icon={<PlusOutlined />} onClick={onAdd}>
          添加员工
        </Button>
      </Space>
    </div>
    );
};