import React from "react";
import { Row, Col, Statistic, Tooltip } from "antd";
import {
  UserOutlined,
  TeamOutlined,
  IdcardOutlined,
  InfoCircleOutlined,
} from "@ant-design/icons";
import DashboardCard from "../../../components/presentation/dashboard-card";
import { theme } from "antd";

interface PositionStats {
  total: number;
  distribution: any[];
  topPosition: { name: string; count: number } | null;
  vacantPositions: number;
}

interface StatisticCardsProps {
  staffs: any[] | undefined;
  departments: any[] | undefined;
  positionStats: PositionStats;
}

export default function StatisticCards({
  staffs,
  departments,
  positionStats,
}: StatisticCardsProps): React.ReactElement {
  const { token } = theme.useToken();

  return (
    <Row gutter={[16, 16]} className="mb-6">
      <Col xs={24} sm={24} md={8} lg={8} xl={8}>
        <DashboardCard
          title={
            <div className="flex items-center">
              <span>总人员数量</span>
              <Tooltip title="系统中所有员工的总数">
                <InfoCircleOutlined className="ml-2 text-gray-400" />
              </Tooltip>
            </div>
          }
          className="h-32"
        >
          <Statistic
            value={staffs ? staffs.length : 0}
            valueStyle={{ color: token.colorPrimary }}
            prefix={<UserOutlined />}
          />
        </DashboardCard>
      </Col>
      <Col xs={24} sm={24} md={8} lg={8} xl={8}>
        <DashboardCard
          title={
            <div className="flex items-center">
              <span>部门总数</span>
              <Tooltip title="系统中所有部门的数量">
                <InfoCircleOutlined className="ml-2 text-gray-400" />
              </Tooltip>
            </div>
          }
          className="h-32"
        >
          <Statistic
            value={Array.isArray(departments) ? departments.length : 0}
            valueStyle={{ color: token.colorSuccess }}
            prefix={<TeamOutlined />}
          />
        </DashboardCard>
      </Col>
      <Col xs={24} sm={24} md={8} lg={8} xl={8}>
        <DashboardCard
          title={
            <div className="flex items-center">
              <span>岗位分布</span>
              <Tooltip title="员工岗位类型统计">
                <InfoCircleOutlined className="ml-2 text-gray-400" />
              </Tooltip>
            </div>
          }
          className="h-32"
        >
          <div className="flex flex-col">
            <Statistic
              value={positionStats.total || 0}
              valueStyle={{ color: "#722ed1" }}
              prefix={<IdcardOutlined />}
              suffix={`种岗位`}
            />
            {positionStats.topPosition && (
              <div className="mt-2 text-sm text-gray-500">
                最多人数岗位: {positionStats.topPosition.name} (
                {positionStats.topPosition.count}人)
              </div>
            )}
          </div>
        </DashboardCard>
      </Col>
    </Row>
  );
}
