/* eslint-disable no-undef */
/* eslint-disable @typescript-eslint/no-unused-expressions */
/* eslint-disable no-unused-expressions */
import { useState, useEffect, useMemo, useCallback } from "react";
import { AgGridReact } from "ag-grid-react";
import { api, useStaff } from "@nice/client";
import { Button, CascaderProps, message, Modal, Input, Upload } from "antd";
import { areaOptions } from "@web/src/app/main/staffinfo_write/area-options";
import StaffInfoWrite from "@web/src/app/main/staffinfo_write/staffinfo_write.page";
import { utils, writeFile, read } from "xlsx";
import { UploadOutlined } from "@ant-design/icons";

function getAreaName(codes: string[], level?: number): string {
  const result: string[] = [];
  let currentLevel: CascaderProps["options"] = areaOptions;
  for (const code of codes) {
    const found = currentLevel?.find((opt) => opt.value === code);
    if (!found) break;
    result.push(String(found.label));
    currentLevel = found.children || [];
    if (level && result.length >= level) break; // 添加层级控制
  }
  return level
    ? result[level - 1] || ""
    : result.join(" / ") || codes.join("/");
}

// 添加表头提取工具函数
function extractHeaders(columns: any[]): string[] {
  const result: string[] = [];

  const extractHeadersRecursive = (cols: any[]) => {
    cols.forEach((col) => {
      if ("children" in col && col.children) {
        extractHeadersRecursive(col.children);
      } else if (col.headerName) {
        result.push(col.headerName);
      }
    });
  };

  extractHeadersRecursive(columns);
  return result;
}

export default function StaffMessage() {
  const [rowData, setRowData] = useState<any[]>([]);
  const [columnDefs, setColumnDefs] = useState<any[]>([]);
  const [selectedRows, setSelectedRows] = useState<any[]>([]);
  const { useCustomFields, softDeleteByIds } = useStaff();
  const fields = useCustomFields();
  const [isEditModalVisible, setIsEditModalVisible] = useState(false);
  const [currentEditStaff, setCurrentEditStaff] = useState<any>(null);
  const [gridApi, setGridApi] = useState<any>(null);
  const [fileNameVisible, setFileNameVisible] = useState(false);
  const [fileName, setFileName] = useState("");
  const [defaultFileName] = useState(
    `员工数据_${new Date().toISOString().slice(0, 10)}`
  );
  const [importVisible, setImportVisible] = useState(false);

  // 获取数据
  const { data: staffData } = api.staff.findMany.useQuery({
    where: {
      deletedAt: null,
    },
    include: {
      fieldValues: {
        include: {
          // 添加这两个关联字段
          staff: { select: { id: true } }, // 关联员工ID
          field: { select: { id: true } }, // 关联字段ID
        },
      },
      department: true,
    },
  } as any);
  // console.log(staffData);
  const actionColumns = [
    {
      field: "action",
      width: 50,
      checkboxSelection: true,
      headerCheckboxSelection: true,
      pinned: "left",
    },
  ];
  // 新增编辑处理函数
  const handleEdit = useCallback(async () => {
    if (selectedRows.length === 0) return;
    if (selectedRows.length > 1) {
      message.error("只能选择一个员工进行编辑");
      return;
    }
    setCurrentEditStaff(selectedRows[0]);
    setIsEditModalVisible(true);
  }, [selectedRows]);
  // 处理编辑完成
  const handleEditComplete = useCallback(() => {
    setIsEditModalVisible(false);
    setCurrentEditStaff(null);
    // 刷新表格数据
    api.staff.findMany.useQuery();
  }, []);

  // 新增删除处理函数
  const handleDelete = useCallback(async () => {
    if (selectedRows.length === 0) return;
    console.log("待删除的选中行数据:", selectedRows); // 新增调试语句
    try {
      await softDeleteByIds.mutateAsync({
        ids: selectedRows?.map((row) => {
          console.log("当前行ID:", row.id); // 检查每个ID
          return row.id;
        }),
      });
      message.success("删除成功");
    } catch (error) {
      message.error("删除失败");
      console.error("详细错误信息:", error); // 输出完整错误堆栈
    }
    // 重新获取数据或本地过滤已删除项
  }, [selectedRows]);

  // 缓存基础列定义
  const baseColumns = useMemo(
    () => [
      {
        field: "showname",
        headerName: "姓名",
        filter: "agSetColumnFilter",
        pinned: "left",
      },
      {
        field: "deptId",
        headerName: "所属部门",
        valueGetter: (params) => params.data.department?.name,
        filter: "agSetColumnFilter",
      },
    ],
    []
  );

  // 缓存动态列定义
  const dynamicColumns = useMemo(
    () =>
      (fields.data || ([] as any)).map((field) => ({
        field: field.name,
        headerName: field.label || field.name,
        filter: "agSetColumnFilter",
        cellStyle: { whiteSpace: "pre-line" },
        autoHeight: true,
        valueGetter: (params) => {
          // 获取原始值
          const rawValue = params.data.fieldValues?.find(
            (fv: any) => fv.fieldId === field.id
          )?.value;

          // 根据字段类型格式化
          switch (field.type) {
            case "cascader":
              return rawValue ? getAreaName(rawValue.split("/")) : "";

            case "date":
              // 格式化日期（假设存储的是ISO字符串）
              return rawValue ? new Date(rawValue).toLocaleDateString() : "";

            case "textarea":
              // 换行处理

              return rawValue?.replace(/,/g, "\n");

            default:
              return rawValue;
          }
        },
      })),
    [fields.data]
  );
  // 合并列定义
  useEffect(() => {
    setColumnDefs([...actionColumns, ...baseColumns, ...dynamicColumns]);
  }, [baseColumns, dynamicColumns]);

  // 更新行数据
  useEffect(() => {
    staffData && setRowData(staffData);
  }, [staffData]);

  // 修改导出模板处理函数
  const handleExportTemplate = useCallback(() => {
    const headerNames = extractHeaders(columnDefs);

    // 创建示例数据行
    const exampleRow: Record<string, string> = {};
    // 定义 fieldsList（移到这里）
    const fieldsList = Array.isArray(fields?.data) ? fields.data : [];

    // 检查是否有选中行
    if (selectedRows.length > 0) {
      // 使用第一条选中的记录作为模板数据
      const templateData = selectedRows[0];

      // 基础字段
      exampleRow["姓名"] = templateData.showname || "";
      exampleRow["所属部门"] = templateData.department?.name || "";

      // 处理自定义字段
      fieldsList.forEach((field: any) => {
        const fieldValue = templateData.fieldValues?.find(
          (fv: any) => fv.fieldId === field.id
        )?.value;

        let displayValue = fieldValue;

        // 根据字段类型处理值
        if (field.type === "cascader" && fieldValue) {
          displayValue = getAreaName(fieldValue.split("/"));
        } else if (field.type === "date" && fieldValue) {
          displayValue = new Date(fieldValue).toLocaleDateString();
        } else if (field.type === "textarea" && fieldValue) {
          displayValue = fieldValue.replace(/,/g, "\n");
        }

        exampleRow[field.label || field.name] = displayValue || "";
      });
    } else {
      // 如果没有选中行，使用默认示例数据
      exampleRow["姓名"] = "张三";
      exampleRow["所属部门"] = "技术部";

      // 添加所有自定义字段的空值
      fieldsList.forEach((field: any) => {
        exampleRow[field.label || field.name] = "";
      });
    }

    // 创建空白行供用户填写
    const emptyRow = headerNames.reduce(
      (obj, header) => {
        obj[header] = "";
        return obj;
      },
      {} as Record<string, string>
    );

    // 创建工作簿和工作表
    const wb = utils.book_new();
    const ws = utils.json_to_sheet([exampleRow], { header: headerNames });

    // 设置列宽
    const colWidth = headerNames.map(() => ({ wch: 20 }));
    ws["!cols"] = colWidth;

    // 在第二行添加提示文字
    const rowIdx = 2; // 第二行索引
    const cellRef = utils.encode_cell({ r: rowIdx, c: 0 }); // A3单元格

    const tipText =
      selectedRows.length > 0
        ? "以上为选中人员数据，请在下方行填写实际数据"
        : "以上为示例数据，请在下方行填写实际数据";

    ws[cellRef] = { t: "s", v: tipText };

    // 手动添加空白行
    utils.sheet_add_json(ws, [emptyRow], {
      skipHeader: true,
      origin: rowIdx + 1,
    });

    // 合并提示文字单元格
    if (!ws["!merges"]) ws["!merges"] = [];
    ws["!merges"].push({
      s: { r: rowIdx, c: 0 }, // 起始单元格 A3
      e: { r: rowIdx, c: Math.min(5, headerNames.length - 1) }, // 结束单元格，跨越多列
    });

    utils.book_append_sheet(wb, ws, "员工模板");
    writeFile(wb, `员工数据模板_${new Date().toISOString().slice(0, 10)}.xlsx`);
  }, [columnDefs, selectedRows, fields.data]);

  // 导出数据处理函数
  const handleConfirm = useCallback(() => {
    setFileNameVisible(true);
  }, []);

  // 处理文件名确认
  const handleFileNameConfirm = useCallback(() => {
    setFileNameVisible(false);
    if (!gridApi) {
      console.error("Grid API 未正确初始化");
      return;
    }

    try {
      // 获取所有数据
      const allData = selectedRows.length > 0 ? selectedRows : rowData;

      // 格式化数据
      const exportData = allData.map((row) => {
        const formattedRow: Record<string, any> = {};

        // 基础字段
        formattedRow["姓名"] = row.showname || "";
        formattedRow["所属部门"] = row.department?.name || "";

        // 动态字段
        const fieldsList = Array.isArray(fields?.data) ? fields.data : [];
        fieldsList.forEach((field: any) => {
          const fieldValue = row.fieldValues?.find(
            (fv: any) => fv.fieldId === field.id
          )?.value;
          let displayValue = fieldValue;

          // 根据字段类型处理值
          if (field.type === "cascader" && fieldValue) {
            displayValue = getAreaName(fieldValue.split("/"));
          } else if (field.type === "date" && fieldValue) {
            displayValue = new Date(fieldValue).toLocaleDateString();
          } else if (field.type === "textarea" && fieldValue) {
            displayValue = fieldValue.replace(/,/g, "\n");
          }

          formattedRow[field.label || field.name] = displayValue || "";
        });

        return formattedRow;
      });

      // 生成工作表
      const ws = utils.json_to_sheet(exportData);
      const wb = utils.book_new();
      utils.book_append_sheet(wb, ws, "员工数据");

      // 生成文件名
      const finalFileName = fileName || defaultFileName;
      writeFile(wb, `${finalFileName}.xlsx`);
    } catch (error) {
      console.error("导出失败:", error);
      message.error("导出失败，请稍后重试");
    }
  }, [fileName, defaultFileName, rowData, selectedRows, fields.data, gridApi]);

  // 获取部门数据
  const { data: departmentsData } = api.department.findMany.useQuery({
    where: { deletedAt: null },
    select: { id: true, name: true },
  });

  // 处理导入数据
  const createMany = api.staff.create.useMutation({
    onSuccess: () => {
      message.success("员工数据导入成功");
      // 不要在这里直接调用 useQuery 钩子
      // 而是使用 trpc 的 invalidate 方法来刷新数据
      api.useContext().staff.findMany.invalidate();
      setImportVisible(false);
    },
    onError: (error) => {
      message.error(`导入失败: ${error.message}`);
    },
  });

  // 处理Excel导入数据
  const handleImportData = useCallback(
    (excelData: any[]) => {
      if (excelData.length === 0) {
        message.warning("没有可导入的数据");
        return;
      }

      try {
        // 确保字段数据有效，提取所有有效的字段ID
        const fieldsList = Array.isArray(fields?.data) ? fields.data : [];
        console.log(
          "可用字段列表:",
          fieldsList.map((f) => ({ id: f.id, name: f.name, label: f.label }))
        );

        // 将Excel数据转换为API需要的格式
        const staffImportData = excelData.map((row, rowIndex) => {
          console.log(`正在处理第${rowIndex + 1}行数据:`, row);

          const staff: any = {
            // 设置必要的字段
            showname: row["姓名"] ? String(row["姓名"]) : "未命名",
            // 避免使用自动生成的字段
            fieldValues: [],
          };

          // 处理部门关联
          if (row["所属部门"] && departmentsData) {
            const deptName = row["所属部门"];
            const matchedDept = departmentsData.find(
              (dept) => dept.name === deptName
            );

            if (matchedDept) {
              staff.department = {
                connect: { id: matchedDept.id },
              };
            } else {
              console.warn(`未找到匹配的部门: ${deptName}`);
            }
          }

          console.log(`准备创建员工: ${staff.showname}`);
          return staff;
        });

        // 逐条导入数据
        if (staffImportData.length > 0) {
          let importedCount = 0;
          const totalCount = staffImportData.length;

          staffImportData.forEach((staffData, index) => {
            createMany.mutate(
              { data: staffData },
              {
                onSuccess: (data) => {
                  console.log(`员工创建成功:`, data);
                  importedCount++;

                  // 显示进度信息
                  if (importedCount === totalCount) {
                    message.success(`所有${totalCount}条数据导入完成`);
                    // 在所有数据导入完成后刷新查询
                    api.useContext().staff.findMany.invalidate();
                  } else {
                    message.info(
                      `已导入 ${importedCount}/${totalCount} 条数据`
                    );
                  }
                },
                onError: (error) => {
                  message.error(
                    `导入第${index + 1}条数据失败: ${error.message}`
                  );
                  console.error(`导入失败的详细数据:`, staffData);
                },
              }
            );
          });

          message.info(`开始导入${staffImportData.length}条员工数据...`);
        }
      } catch (error) {
        console.error("处理导入数据失败:", error);
        message.error("数据格式错误，导入失败");
      }
    },
    [fields.data, createMany, departmentsData]
  );

  return (
    <>
      <div className="flex justify-between items-center mb-4">
        <div className="space-x-2">
          <Button
            className="bg-red-500 hover:bg-red-600 border-red-500 text-white rounded-md px-4 py-2"
            onClick={handleDelete}
          >
            批量删除
          </Button>
          <Button
            className="bg-blue-500 hover:bg-blue-600 border-blue-500 text-white rounded-md px-4 py-2"
            onClick={handleEdit}
          >
            编辑
          </Button>
          <Button
            onClick={() => setImportVisible(true)}
            className="bg-orange-500 text-white px-4 py-2 rounded hover:bg-orange-600"
          >
            导入Excel
          </Button>
          <Button
            onClick={handleExportTemplate}
            className="bg-blue-500 text-white px-4 py-2 rounded hover:bg-blue-600"
          >
            导出模板
          </Button>
          <Button
            onClick={handleConfirm}
            className="bg-green-500 text-white px-4 py-2 rounded hover:bg-green-600"
          >
            导出所有数据
          </Button>
        </div>
        <Button
          className="bg-gray-100 hover:bg-gray-200 border-gray-300 text-gray-700 rounded-md px-4 py-2"
          onClick={() => {}}
        >
          调试数据
        </Button>
      </div>

      <div className="bg-white rstaffDataounded-lg shadow-md p-6 mb-4">
        <h1 className="text-2xl font-bold text-gray-800 mb-4">人员总览</h1>

        <div className="ag-theme-alpine w-full min-h-[480px] h-auto overflow-visible">
          <AgGridReact
            rowData={rowData}
            columnDefs={columnDefs}
            pagination={true}
            paginationPageSize={10}
            paginationPageSizeSelector={[10, 20, 50, 100]}
            onSelectionChanged={(e) => setSelectedRows(e.api.getSelectedRows())}
            rowSelection="multiple"
            className="rounded border border-gray-200"
            headerHeight={40}
            rowHeight={40}
            domLayout="autoHeight"
            onGridReady={(params) => setGridApi(params.api)}
          />
        </div>
      </div>

      {/* 编辑弹窗 */}
      <Modal
        title="编辑员工信息"
        open={isEditModalVisible}
        onCancel={() => setIsEditModalVisible(false)}
        footer={null}
        width={1000}
        destroyOnClose={true}
      >
        {currentEditStaff && (
          <StaffInfoWrite
            staffId={currentEditStaff.id}
            initialData={currentEditStaff}
            fieldValues={currentEditStaff.fieldValues}
            onComplete={handleEditComplete}
            setIsEditModalVisible={setIsEditModalVisible}
          />
        )}
      </Modal>

      {/* 导出文件名对话框 */}
      <Modal
        title="输入文件名"
        open={fileNameVisible}
        onOk={handleFileNameConfirm}
        onCancel={() => setFileNameVisible(false)}
        okText="导出"
        cancelText="取消"
      >
        <Input
          placeholder={`默认名称: ${defaultFileName}`}
          value={fileName}
          onChange={(e) => setFileName(e.target.value)}
        />
      </Modal>

      {/* 导入对话框 */}
      <Modal
        title="导入员工数据"
        open={importVisible}
        onCancel={() => setImportVisible(false)}
        footer={null}
      >
        <Upload
          accept=".xlsx,.xls"
          beforeUpload={(file) => {
            const reader = new FileReader();
            reader.onload = (e) => {
              try {
                const wb = read(e.target?.result);
                const data = utils.sheet_to_json(wb.Sheets[wb.SheetNames[0]]);
                if (data.length === 0) {
                  message.warning("Excel文件中没有数据");
                  return;
                }
                message.info(`读取到${data.length}条数据，正在处理...`);
                handleImportData(data);
              } catch (error) {
                console.error("解析Excel文件失败:", error);
                message.error("Excel文件格式错误，请确保使用正确的模板");
              }
            };
            reader.readAsArrayBuffer(file);
            return false;
          }}
        >
          <Button icon={<UploadOutlined />}>选择Excel文件</Button>
        </Upload>
        <div className="mt-4 text-gray-500 text-sm">
          提示：请使用导出模板功能获取标准模板，按格式填写数据后导入
        </div>
      </Modal>
    </>
  );
}
